using System;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.Xml.Serialization; 
using Toub.Sound.Midi;

namespace JunoComposer {

public enum Midmod :byte {run,stop,rewind};

public class MainForm : Form {
	
	// fields...........................
	static string progdir=System.IO.Directory.GetCurrentDirectory();
	int widform=800, heiform=600-24;
	int xpi=8, ypi=60, widpi=768, heipi=72,  // pi=Piano
	keypixpi=16, lastinstkeypixpi=16,	octpi=3, oldoctpi=3, lastinstoctpi=3, keynr=0;
	static internal byte pgm=0, oldpgm=0, lastpgmbeforedrums=0;
	static internal byte chn=0, oldchn=0, barsdisplay=4;
	byte lastdrum=31; 
	byte skin =1;
	static internal int xlastkey=0; static internal int colorlastkey=0;

	internal bool rytmreplaying=true;
	static internal int tic=0, bpm=120, oldbpm=120, tpb=12; // midi-tics per beat (beat=metronomeclick)

	static internal long tim1, checks; 
	static internal int xwoe=8, xwoi=24, ywo=280, widwo=768, heiwo=256, colpix=8; //wo=Workbench
	static internal int prewo=xwoi-xwoe, colswo=widwo/colpix, linheiwo=16, linswo=heiwo/linheiwo,
		colwo=0, linwo=0;
	static internal int[,] ryt=new int[linswo,colswo];
	static internal int[] rychoi=new int[linswo];
	static internal byte maxdrumsounds=88;
	static internal Point poi=new System.Drawing.Point(prewo+colpix,ywo-20);
	
	string[] igroup=new string[20]; // groups of instruments
	string[,] inst=new string[18,8]; // instruments
	string[] rytinst=new string[maxdrumsounds]; // instruments
	string[] rytquick=new string[maxdrumsounds]; // instruments
	
	Font myfont =new Font("Tahoma",8);
	Font myfont7 =new Font("Tahoma",7);
	SolidBrush sbblack=new SolidBrush(Color.Black);
  SolidBrush sbivory=new SolidBrush(q.color_input);
 	SolidBrush sbcontrol=new SolidBrush(q.color_control);
 	SolidBrush sbyellow=new SolidBrush(Color.Yellow);
	SolidBrush sbgray=new SolidBrush(Color.Gray);
  SolidBrush sborange=new SolidBrush(Color.Orange);
	SolidBrush sbred=new SolidBrush(Color.Red);
	SolidBrush sbgreen=new SolidBrush(Color.Green);
	SolidBrush sbpaper=new SolidBrush(q.color_paper);
  SolidBrush sbback=new SolidBrush(q.color_back);
  Pen penblack=new Pen(Color.Blue,1);
  Pen penblue=new Pen(Color.Blue,1);
	Pen penbluedot=new Pen(Color.Blue,1);
	Pen penbluedash=new Pen(Color.Blue,1);
	//Pen penred=new Pen(Color.Red,2);
	Pen penhigh=new Pen(q.color_high,2);
	Pen peninput=new Pen(q.color_input,2);
	Pen penx=new Pen(Color.Yellow,1);

	// a) create some controls..........
	private Timer timer1;
	private Button btnew, btload, btrun, btstop, btrewind, btsave, btinfo, btoff, btskin, btpi, btdrums, btimp;
	private Label lbsong, lbkbzoom, lbspeed, lboctave, lbbars, lbgroup, lbinstrum,lbbeatmark;
	internal static TextBox tb1,tb2; 
		private TextBox tb3, tb4, tbrytname, tbspeed;
	internal TrackBar trkbzoom, trspeed, troctave, trbars;
	private ComboBox cbgroup, cbinstrum;

	//private System.Windows.Forms.PictureBox pictureBox1;

	public MainForm() {
		initInstNames();
		InitializeComponents();
		cbgroup.SelectedIndex=0; this.Refresh();
		tim1=System.DateTime.Now.Ticks; checks=0;
	}

	
	void InitializeComponents() {
						//this.SuspendLayout();
		// define mainform..............................
		this.Name = "MainForm";
		this.Text = "Juno Compose 0.24";
		this.BackColor=q.color_back;
		this.Font = new System.Drawing.Font("Tahoma", 11);
		this.Size = new System.Drawing.Size(widform, heiform);
		this.Click += new System.EventHandler(f1_Click);
		this.Paint += new PaintEventHandler(f1_paint);
		this.Paint += new PaintEventHandler(drawrytNames);
		this.Paint += new PaintEventHandler(drawMyGrid);
		//this.Opacity=0.7;

		// timer
		timer1 = new Timer();
		timer1.Interval = 10;
		timer1.Enabled = true;
		timer1.Tick += new EventHandler (timer1_jobx);

		//  Set up member lbsong
		lbsong = new System.Windows.Forms.Label();
		lbsong.Name = "lbsong";
		lbsong.BackColor = q.color_paper;
		lbsong.Location = new System.Drawing.Point(64+48, 32-3);
		lbsong.Size = new System.Drawing.Size(widpi/2-8, 24);
		lbsong.TabIndex = 5;
		lbsong.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
		lbsong.Font = new System.Drawing.Font("Tahoma", 8);
		lbsong.Text = "(Song)";
		this.Controls.Add(lbsong);

		//  Set up member lbbeatmark
		lbbeatmark = new System.Windows.Forms.Label();
		lbbeatmark.Name = "lbbeatmark";
		lbbeatmark.BackColor = q.color_back;
		lbbeatmark.ForeColor = q.color_high;
		lbbeatmark.Location = poi;
		lbbeatmark.Size = new System.Drawing.Size(8, 8);
		lbbeatmark.TabIndex = 5;
		lbbeatmark.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
		lbbeatmark.Font = new System.Drawing.Font("Tahoma", 8);
		lbbeatmark.Text = "V";
		this.Controls.Add(lbbeatmark);

		//  Set up member tb1
		tb1 = new System.Windows.Forms.TextBox();
		tb1.Name = "tb1";
		tb1.Font = new System.Drawing.Font("Tahoma", 8);
		tb1.Text = "";
		tb1.BackColor = q.color_input;
		tb1.Size = new System.Drawing.Size(24, 20);
		tb1.Location = new System.Drawing.Point(595, ypi+heipi+8);
		tb1.TabIndex = 9;
		this.Controls.Add(tb1);
		
		//  Set up member tb2
		tb2 = new System.Windows.Forms.TextBox();
		tb2.Name = "tb2";
		tb2.Font = new System.Drawing.Font("Tahoma", 8);
		tb2.Text = "";
		tb2.BackColor = q.color_input;
		tb2.Size = new System.Drawing.Size(24, 20);
		tb2.Location = new System.Drawing.Point(630, ypi+heipi+8);
		tb2.TabIndex = 10;
		this.Controls.Add(tb2);

		//  Set up member tb3
		tb3 = new System.Windows.Forms.TextBox();
		tb3.Name = "tb3";
		tb3.Font = new System.Drawing.Font("Tahoma", 8);
		tb3.Text = "";
		tb3.BackColor = q.color_input;
		tb3.Size = new System.Drawing.Size(24, 20);
		tb3.Location = new System.Drawing.Point(665, ypi+heipi+8);
		tb3.TabIndex = 10;
		this.Controls.Add(tb3);

		//  Set up member tb4
		tb4 = new System.Windows.Forms.TextBox();
		tb4.Name = "tb4";
		tb4.Font = new System.Drawing.Font("Tahoma", 8);
		tb4.Text = "";
		tb4.BackColor = q.color_input;
		tb4.Size = new System.Drawing.Size(24, 20);
		tb4.Location = new System.Drawing.Point(700, ypi+heipi+8);
		tb4.TabIndex = 10;
		this.Controls.Add(tb4);


// ########################################################################################


		//  Set up member lbkbzoom
		lbkbzoom = new System.Windows.Forms.Label();
		lbkbzoom.Name = "lbkbzoom";
		lbkbzoom.BackColor = q.color_control;
		lbkbzoom.Location = new System.Drawing.Point(xpi+56+48, ypi+heipi+8);
		lbkbzoom.Size = new System.Drawing.Size(100, 16);
		//lbkbzoom.TabIndex = 5;
		lbkbzoom.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
		lbkbzoom.Font = new System.Drawing.Font("Tahoma", 8);
		lbkbzoom.Text = "KeyboardZoom";
		this.Controls.Add(lbkbzoom);

		//  Set up member trkbzoom (KeyboardZoom)
		trkbzoom = new System.Windows.Forms.TrackBar();
		trkbzoom.Name = "trkbzoom";
		trkbzoom.BackColor=Color.LightSteelBlue;
		//trkbzoom.TabIndex = 1;
		trkbzoom.Value =3;
		trkbzoom.Maximum = 5;
		trkbzoom.AutoSize=false;
		trkbzoom.Location = new System.Drawing.Point(xpi+56+48, ypi+heipi+24);
		trkbzoom.Size = new System.Drawing.Size(100, 28);
		this.Controls.Add(trkbzoom);
		trkbzoom.Scroll += new System.EventHandler(trkbzoom_Scroll);
		
		//  Set up member lbspeed
		lbspeed = new System.Windows.Forms.Label();
		lbspeed.Name = "lbspeed";
		lbspeed.BackColor = q.color_control;
		lbspeed.Location = new System.Drawing.Point(xpi+56+48, ypi+heipi+8);
		lbspeed.Size = new System.Drawing.Size(100, 24);
		//lbspeed.TabIndex = 5;
		lbspeed.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
		lbspeed.Font = new System.Drawing.Font("Tahoma", 8);
		lbspeed.Text = "Speed (Beats/Min)";
		this.Controls.Add(lbspeed);

		//  Set up member trspeed (KeyboardZoom)
		trspeed = new System.Windows.Forms.TrackBar();
		trspeed.Name = "trspeed";
		trspeed.BackColor=Color.LightSteelBlue;
		//trspeed.TabIndex = 1;
		trspeed.TickFrequency=1;
		trspeed.Minimum = 1;
		trspeed.Maximum = 300;
		trspeed.Value =120;
		trspeed.AutoSize=false;
		trspeed.Location = new System.Drawing.Point(xpi+24+56+48, ypi+heipi+24);
		trspeed.Size = new System.Drawing.Size(76, 28);
		this.Controls.Add(trspeed);
		trspeed.Scroll += new System.EventHandler(trspeed_Scroll);
		
		//  Set up member tbspeed
		tbspeed = new System.Windows.Forms.TextBox();
		tbspeed.Name = "tbspeed";
		tbspeed.Font = new System.Drawing.Font("Tahoma", 8);
		tbspeed.Text = "120";
		tbspeed.BackColor = q.color_input;
		tbspeed.Location = new System.Drawing.Point(xpi+56+48, ypi+heipi+32);
		tbspeed.Size = new System.Drawing.Size(24, 28);
		this.Controls.Add(tbspeed);
		tbspeed.Enter +=new EventHandler(tbspeed_cr);
		tbspeed.KeyUp += new KeyEventHandler(tbspeed_okp);
		tbspeed.LostFocus +=new EventHandler(tbspeed_lf);
		
		//  Set up member lboctave
		lboctave = new System.Windows.Forms.Label();
		lboctave.Name = "lboctave";
		lboctave.BackColor = q.color_control;
		lboctave.Location = new System.Drawing.Point(xpi+108+56+48, ypi+heipi+8);
		lboctave.Size = new System.Drawing.Size(100, 16);
		//lboctave.TabIndex = 5;
		lboctave.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
		lboctave.Font = new System.Drawing.Font("Tahoma", 8);
		lboctave.Text = "Octave";
		this.Controls.Add(lboctave);

		//  Set up member troctave (octave)
		troctave = new System.Windows.Forms.TrackBar();
		troctave.Name = "troctave";
		troctave.BackColor=Color.LightSteelBlue;
		//troctave.TabIndex = 1;
		troctave.Value =2;
		troctave.Maximum = 3;
		troctave.AutoSize=false;
		troctave.Location = new System.Drawing.Point(xpi+108+56+48, ypi+heipi+24);
		troctave.Size = new System.Drawing.Size(100, 28);
		this.Controls.Add(troctave);
		troctave.Scroll += new System.EventHandler(troctave_Scroll);

		//  Set up member lbbars
		lbbars = new System.Windows.Forms.Label();
		lbbars.Name = "lbbars";
		lbbars.BackColor = q.color_control;
		lbbars.Location = new System.Drawing.Point(xpi+108+56+48, ypi+heipi+8);
		lbbars.Size = new System.Drawing.Size(100, 16);
		//lbbars.TabIndex = 5;
		lbbars.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
		lbbars.Font = new System.Drawing.Font("Tahoma", 8);
		lbbars.Text = "Bars-Display";
		this.Controls.Add(lbbars);

		//  Set up member trbars (octave)
		trbars = new System.Windows.Forms.TrackBar();
		trbars.Name = "trbars";
		trbars.BackColor=Color.LightSteelBlue;
		trbars.Value =3;
		trbars.Text="abc";
		trbars.Maximum = 7;
		trbars.Minimum = 0;
		trbars.AutoSize=false;
		trbars.Location = new System.Drawing.Point(xpi+108+56+48, ypi+heipi+24);
		trbars.Size = new System.Drawing.Size(100, 28);
		this.Controls.Add(trbars);
		trbars.Scroll += new System.EventHandler(trbars_Scroll);


// #################################################################################


		//  Set up member lbgroup
		lbgroup = new System.Windows.Forms.Label();
		lbgroup.Name = "lbgroup";
		lbgroup.BackColor = q.color_control;
		lbgroup.Location = new System.Drawing.Point(xpi+216+56+48, ypi+heipi+8);
		lbgroup.Size = new System.Drawing.Size(100, 24);
		//lboctave.TabIndex = 5;
		lbgroup.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
		lbgroup.Font = new System.Drawing.Font("Tahoma", 8);
		lbgroup.Text = "Group of Instr.";
		this.Controls.Add(lbgroup);

		//  Set up cbgroup
		cbgroup = new System.Windows.Forms.ComboBox();
		cbgroup.Name = "cbgroup";
		cbgroup.Font = new System.Drawing.Font("Tahoma", 8);
		cbgroup.BackColor=q.color_input;
		cbgroup.Text=igroup[1]; //1=pianos
		cbgroup.Location = new System.Drawing.Point(xpi+216+56+48, ypi+heipi+30);
		cbgroup.Size = new System.Drawing.Size(100,24);
		cbgroup.MaxDropDownItems=17;
		this.Controls.Add(cbgroup);
		cbgroup.Items.AddRange(new object[] {
			igroup[0],igroup[1],igroup[2],igroup[3],igroup[4],igroup[5],igroup[6],igroup[7],
			igroup[8],igroup[9],igroup[10],igroup[11],igroup[12],igroup[13],igroup[14],igroup[15],
			igroup[16]});
		cbgroup.SelectedIndexChanged += new System.EventHandler(this.cbgroup_SIC);
		
		//  Set up member lbinstrum
		lbinstrum = new System.Windows.Forms.Label();
		lbinstrum.Name = "lbinstrum";
		lbinstrum.BackColor = q.color_control;
		lbinstrum.Location = new System.Drawing.Point(xpi+324+56+48, ypi+heipi+8);
		lbinstrum.Size = new System.Drawing.Size(100, 24);
		//lbinstrum.TabIndex = 5;
		lbinstrum.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
		lbinstrum.Font = new System.Drawing.Font("Tahoma", 8);
		lbinstrum.Text = "Instrument";
		this.Controls.Add(lbinstrum);

		//  Set up cbinstrum
		cbinstrum = new System.Windows.Forms.ComboBox();
		cbinstrum.Name = "cbinstrum";
		cbinstrum.BackColor=q.color_input;
		cbinstrum.Font = new System.Drawing.Font("Tahoma", 8);
		cbinstrum.Text=inst[0,0];
		cbinstrum.Size = new System.Drawing.Size(100, 24);
		cbinstrum.Location = new System.Drawing.Point(xpi+324+56+48, ypi+heipi+31);
		cbinstrum.MaxDropDownItems=9;
		this.Controls.Add(cbinstrum);
		// just for the first run fill it with items of group 1...........
		cbinstrum.Items.AddRange(new object[] {
			inst[0,0],inst[0,1],inst[0,2],inst[0,3],inst[0,4],inst[0,5],inst[0,6],inst[0,7]});
		cbinstrum.SelectedItem=0;
		cbinstrum.SelectedIndexChanged += new System.EventHandler(this.cbinstrum_SIC);
		
		//  Set up member tbrytname
		tbrytname = new System.Windows.Forms.TextBox();
		tbrytname.Name = "tbrytname";
		tbrytname.Font = new System.Drawing.Font("Tahoma", 8);
		tbrytname.Text = "";
		tbrytname.BackColor = q.color_input;
		tbrytname.Size = new System.Drawing.Size(100, 24);
		//tbrytname.Location = new System.Drawing.Point(656,56);
		tbrytname.Location = new System.Drawing.Point(xpi+324+56+48, ypi+heipi+31);
		tbrytname.TabIndex = 10;
		tbrytname.Visible=false;
		this.Controls.Add(tbrytname);

// ###################################################################################
		
		
		//  Set up member btinfo
		btinfo = new System.Windows.Forms.Button();
		btinfo.Name = "btinfo";
		btinfo.Font = new System.Drawing.Font("Tahoma", 7);
		btinfo.BackColor = System.Drawing.Color.PowderBlue;
		btinfo.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btinfo.TabIndex = 0;
		btinfo.Text = "?";
		btinfo.Size = new System.Drawing.Size(48, 22);
		btinfo.Location = new System.Drawing.Point(10, 4);
		this.Controls.Add(btinfo);
		btinfo.Click += new System.EventHandler(btinfo_Click);

		//  Set up member btnew
		btnew = new System.Windows.Forms.Button();
		btnew.Name = "btnew";
		btnew.Font = new System.Drawing.Font("Tahoma", 11);
		btnew.BackColor = System.Drawing.Color.PowderBlue;
		btnew.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btnew.TabIndex = 0;
		btnew.Text = "New";
		btnew.Size = new System.Drawing.Size(64, 22);
		btnew.Location = new System.Drawing.Point(64+48, 4);
		this.Controls.Add(btnew);
		btnew.Click += new System.EventHandler(btnew_Click);
		
		//  Set up member btload
		btload = new System.Windows.Forms.Button();
		btload.Name = "btload";
		btload.Font = new System.Drawing.Font("Tahoma", 11);
		btload.BackColor = System.Drawing.Color.PowderBlue;
		btload.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btload.TabIndex = 1;
		btload.Text = "Load";
		btload.Size = new System.Drawing.Size(64, 22);
		btload.Location = new System.Drawing.Point(136+48, 4);
		this.Controls.Add(btload);
		btload.Click += new System.EventHandler(btload_Click);	
		
		//  Set up member btsave
		btsave = new System.Windows.Forms.Button();
		btsave.Name = "btsave";
		btsave.Font = new System.Drawing.Font("Tahoma", 11);
		btsave.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btsave.TabIndex = 8;
		btsave.Text = "Save";
		btsave.Size = new System.Drawing.Size(64, 22);
		btsave.Location = new System.Drawing.Point(208+48, 4);
		this.Controls.Add(btsave);
		btsave.Click += new System.EventHandler(btsave_Click);	
		
		//  Set up member btrun
		btrun = new System.Windows.Forms.Button();
		btrun.Name = "btrun";
		btrun.Font = new System.Drawing.Font("Tahoma", 11);
		btrun.BackColor = q.color_control;
		btrun.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btrun.TabIndex = 2;
		btrun.Text = ">";
		btrun.Size = new System.Drawing.Size(32, 22);
		btrun.Location = new System.Drawing.Point(378, 4);
		this.Controls.Add(btrun);
		btrun.Click += new System.EventHandler(btrun_Click);

		//  Set up member btstop
		btstop = new System.Windows.Forms.Button();
		btstop.Name = "btstop";
		btstop.Font = new System.Drawing.Font("Tahoma", 11);
		btstop.BackColor = q.color_control;
		btstop.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btstop.TabIndex = 3;
		btstop.Text = "||";
		btstop.Size = new System.Drawing.Size(32, 22);
		btstop.Location = new System.Drawing.Point(418, 4);
		this.Controls.Add(btstop);
		btstop.Click += new System.EventHandler(btstop_Click);
		
		//  Set up member btrewind
		btrewind = new System.Windows.Forms.Button();
		btrewind.Name = "btrewind";
		btrewind.BackColor = q.color_control;
		btrewind.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btrewind.TabIndex = 4;
		btrewind.Text = "|<";
		btrewind.Size = new System.Drawing.Size(32, 22);
		btrewind.Location = new System.Drawing.Point(458, 4);
		this.Controls.Add(btrewind);
		btrewind.Click += new System.EventHandler(btrewind_Click);
		
		//  Set up member btoff
		btoff = new System.Windows.Forms.Button();
		btoff.Name = "btoff";
		btoff.Font = new System.Drawing.Font("Tahoma", 8);
		btoff.BackColor = System.Drawing.Color.PowderBlue;
		btoff.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btoff.TabIndex = 0;
		btoff.Text = "off";
		btoff.Size = new System.Drawing.Size(40, 22);
		btoff.Location = new System.Drawing.Point(xpi+widpi-40, ypi+heipi+8+40);
		this.Controls.Add(btoff);
		btoff.Visible=false;
		btoff.Click += new System.EventHandler(btoff_Click);

		//  Set up member btskin
		btskin = new System.Windows.Forms.Button();
		btskin.Name = "btskin";
		btskin.Font = new System.Drawing.Font("Tahoma", 8);
		btskin.BackColor = System.Drawing.Color.PowderBlue;
		btskin.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btskin.TabIndex = 0;
		btskin.Text = "Skin>";
		btskin.Size = new System.Drawing.Size(40, 22);
		btskin.Location = new System.Drawing.Point(xpi+widpi-40, 4);
		this.Controls.Add(btskin);
		btskin.Click += new System.EventHandler(btskin_Click);

		//  Set up member btpi
		btpi = new System.Windows.Forms.Button();
		btpi.Name = "btpi";
		btpi.Font = new System.Drawing.Font("Tahoma", 8);
		btpi.BackColor = System.Drawing.Color.PowderBlue;
		btpi.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btpi.TabIndex = 0;
		btpi.Text = "Piano";
		btpi.Size = new System.Drawing.Size(48, 22);
		btpi.Location = new System.Drawing.Point(10, ypi+heipi+8+24);
		this.Controls.Add(btpi);
		btpi.Click += new System.EventHandler(btpi_Click);
	
		//  Set up member btdrums
		btdrums = new System.Windows.Forms.Button();
		btdrums.Name = "btdrums";
		btdrums.Font = new System.Drawing.Font("Tahoma", 8);
		btdrums.BackColor = System.Drawing.Color.PowderBlue;
		btdrums.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btdrums.TabIndex = 0;
		btdrums.Text = "Drums";
		btdrums.Size = new System.Drawing.Size(48, 22);
		btdrums.Location = new System.Drawing.Point(10, ypi+heipi+8);
		this.Controls.Add(btdrums);
		btdrums.Click += new System.EventHandler(btdrums_Click);
	
		//  Set up member btimp
		btimp = new System.Windows.Forms.Button();
		btimp.Name = "btimp";
		btimp.Font = new System.Drawing.Font("Tahoma", 8);
		btimp.BackColor = System.Drawing.Color.PowderBlue;
		btimp.FlatStyle = System.Windows.Forms.FlatStyle.System;
		btimp.TabIndex = 0;
		btimp.Text = "imp";
		btimp.Size = new System.Drawing.Size(48,24);
		btimp.Location = new System.Drawing.Point(10, 30);
		this.Controls.Add(btimp);
		btimp.Click += new System.EventHandler(btimp_Click);

		//this.ResumeLayout(false);
		
	}
	
//	void setupmybutton(string nam, string tx, int x, int y, int wd, int hi) {
//		Button bt = new System.Windows.Forms.Button();
//		bt.Name=nam; bt.Text=tx;
//		bt.Location = new System.Drawing.Point(x, y); 
//		bt.Size = new System.Drawing.Size(wd, hi);
//		bt.Font = new System.Drawing.Font("Tahoma", 11);
//		bt.FlatStyle = System.Windows.Forms.FlatStyle.System;
//		//this.Controls.Add(bt);
//		//bt.Click += new System.EventHandler();
//	}

	
	
	// ##########################################################################################
	
	// b) define eventhandlers - say what they shall do ........
	
	private void timer1_jobx(object sender, EventArgs e){
		long tim2=System.DateTime.Now.Ticks;
		float f1=10*1000*1000/tpb/bpm*60; long tickspertic=(long)f1;
		int t, tics=barsdisplay*tpb; 

		if (tim2-tim1-checks*tickspertic>tickspertic) {
			//if (checks % tpb==0) lbsong.Text=((tim2-tim1)/10/1000).ToString();
			// drums-pattern-replay.............
			if (chn==9 && rytmreplaying==true) {
				if (tic % tpb ==tpb/2) tb4.Text="+";
				if (tic % tpb ==0) {
					tb4.Text=(tic/tpb+1).ToString();
					poi.X=prewo+colpix+tic*colpix; poi.Y= ywo-20;
					lbbeatmark.Location = poi; 
				}
				for (int i=0;i<linswo ;i++ ) {
					t=MainForm.ryt[i,tic]; 
					if (t>0 && rytmreplaying==true) midionline.play1tone((byte)rychoi[i]);
				}
				tic++; if (tic==tics) tic=0;
			}
			checks++;
		}
	}

	
	private void btnew_Click(object sender, System.EventArgs e){ // sender and e are tradition
	if (chn==9) {	
		System.Array.Clear(ryt,0,ryt.Length);
		mytimerreset(); rytmreplaying=true;
		initInstNames();
		this.Refresh();
	} 
	}
	
	private void btsave_Click(object sender, System.EventArgs e) {
		string fnam=q.savefileselect("Rhytmpattern | *.ryt; ");
		if (fnam.Length>0) {
			FileStream f = new FileStream(fnam, FileMode.Create);
			StreamWriter sw = new StreamWriter(f);
			string s=""; byte t=0; byte rc=0;
			sw.WriteLine(barsdisplay.ToString());
			sw.WriteLine(bpm.ToString());
			for (int i=0;i<linswo ;i++ ) {
				rc=(byte)rychoi[i];
				//if (rc>0) q.mm(i);
				s=rc.ToString();
				for (int k=0;k<colswo ;k++ ) {
					t=(byte)ryt[i,k]; 
					if (t>0) s=s+",  "+k.ToString()+","+t.ToString();
				}
				sw.WriteLine(s);	//q.mm(s);
			}
			sw.Close(); f.Close();
		}
	}

	private void btload_Click(object sender, System.EventArgs e){ // sender and e are tradition
		string fnam=q.fileselect("Rhytmpattern | *.ryt; ");
		if (fnam.Length>0) {
			lbsong.Text=fnam;
			// reset all ryt-entries..........
			for (int i=0;i<linswo ;i++ ) rychoi[i]=0;
			for (int i=0;i<linswo ;i++ ) {
				for (int k=0;k<colswo ;k++ )	
					ryt[i,k]=0;
			}
			// reading.................................................
			FileStream f = new FileStream(fnam, FileMode.Open);
			StreamReader sr = new StreamReader(f);
			string s=""; 
			s=sr.ReadLine(); barsdisplay=Byte.Parse(s); trbars.Value=barsdisplay-1;
			s=sr.ReadLine(); bpm=Int32.Parse(s); trspeed.Select(); oldbpm=bpm; tbspeed.Text=bpm.ToString();
				trspeed.Value=bpm; mytimerreset();
			byte linnr=0; byte instru=0; byte t=0; byte posi=0, vel=0;
			while ((s=sr.ReadLine())!= null) {
				//q.mm(s);
				char[] separators = {','};
				string[] stemp = s.Split(separators);
				int k=0; 
				foreach (string sub in stemp) {
					t=Byte.Parse(sub); //q.mm("sub="+t+" *2=+"+t*2); 
					if (k==0) {instru=t; rychoi[linnr]=instru; 
						//q.mm("linr="+linnr+" instru="+instru);
					}
					else {
						if ((k-1)%2==0) posi=t;
						if ((k-1)%2==1) {vel=t; ryt[linnr,posi]=vel; }
					}
					k++;	
				}
				linnr++;
			}
			sr.Close(); f.Close();
			this.Refresh(); //... to draw ryr-tags into the grid
		}
	}

	private void btinfo_Click(object sender, System.EventArgs e) {
		MessageBox.Show("\n"+
		"I wrote this simple Midi-Drum-Sequencer as my first exercise after 4 weeks study of C#.Net. \n"+
		"Therefore I used SharpDevelop-Editor 0.92 (Thanks to the smart developer Mike Krüger, Germany). \n\n"+
		"What you can do with it: \n\n"+
		"    [Drums]  Select instruments of the drumkit and enter drumbeats for max. 8 beats. Save / Load - and nothing else. \n\n"+
		"    [Piano]   Switch to other Instruments (piano, flute, trumpet...) of your soundcard. A sequencer is missing. \n\n"+
		"    [imp]      Just load some music-file to play - and nothing else. Displaying its Midi-Events has to be coded soon. \n\n"+
		"For the next step I will extend the Drum-sequencer to an arpeggio-workbench for accord-accompaignements; \n"+
		"Finally I try to write a program for composing music: alterate, calculate and randomize patterns, using musical rules. \n\n"+
		"Thank you for using Toub.Sound.Midi of Stephen Toub stoub@microsoft.com and Interop.QuartzTypeLib.dll °Microsoft. \n"+
		"The crazy coloured background and the piano-keyboard is painted by me. The other elements are copied for free. \n\n"+
		"                             Köln (Cologne/Germany), Dec. 30, 2002   +++ AKL@JunoLand.de ",
		"JunoCompose 0.24 (c) Andreas Klotz  2002");
	}

	private void btrun_Click(object sender, System.EventArgs e){ 
		if (midireplay.lastmidnam.Length>0)
			midireplay.playmid(midireplay.lastmidnam,Midmod.run);
		if (chn==9) {	rytmreplaying=true;	}
	}

	private void btstop_Click(object sender, System.EventArgs e) {
		//q.mm("chn="+chn+" rytmreplaying="+rytmreplaying.ToString());
		if (midireplay.lastmidnam.Length>0)
			midireplay.playmid(midireplay.lastmidnam,Midmod.stop);
		if (chn==9) {
			if (rytmreplaying==false) rytmreplaying=true;	else rytmreplaying=false;
		}
	}

	private void btrewind_Click(object sender, System.EventArgs e) {
		if (midireplay.lastmidnam.Length>0)
			midireplay.playmid(midireplay.lastmidnam,Midmod.rewind);
			if (chn==9) {mytimerreset(); rytmreplaying=true;	}
	}

	private void btoff_Click(object sender, System.EventArgs e) {
		midionline.play1tone(0);
	}

	private void btskin_Click(object sender, System.EventArgs e) {
		if (skin==2) skin=0;
		else skin++;
		this.Refresh();
	}

	private void btpi_Click(object sender, System.EventArgs e){ 
		rytmreplaying=false;
		midionline.play1tone(0);
		if (chn==9) { // from rytm to normal keyboard for instruments
			chn=0; pgm=lastpgmbeforedrums; }
		else {pgm=0;} // from any instrument to instrument=pi
		byte temp=pgm;
		cbgroup.SelectedIndex=pgm / 8 +1;  cbinstrum.SelectedIndex=temp % 8; this.Refresh();
		pgm=temp;
	}

	private void btdrums_Click(object sender, System.EventArgs e){ 
		rytmreplaying=true;
		lastpgmbeforedrums=pgm;
		cbgroup.SelectedIndex=0; 
		this.Refresh();
		//pgm=(byte)0; tb1.Text=pgm.ToString();
	}

	private void btimp_Click(object sender, System.EventArgs e){ 
		string s=q.fileselect("Audio | *.mid; *.mp3; *.wav; |All Files | *.*"); lbsong.Text=s;
		if (s.Length>0) midireplay.playmid(s,Midmod.run);
	}
	
	private void trkbzoom_Scroll(object sender, System.EventArgs e){ 
		int t=trkbzoom.Value; 
		troctave.Visible=true; octpi=oldoctpi;
		switch (t) {
		case 0: keypixpi=6; oldoctpi=octpi; octpi=0; 
				troctave.Visible=false; break;
		case 1: keypixpi=10; break;
		case 2: keypixpi=12; break;
		case 3: keypixpi=16; break;
		case 4: keypixpi=20; break;
		case 5: keypixpi=28; break;
		default: keypixpi=36; break;
		}
		this.Refresh();
	}

	private void trspeed_Scroll(object seffgszqcnder, System.EventArgs e){ 
		bpm=trspeed.Value;
		tbspeed.Text=MainForm.bpm.ToString();
		mytimerreset();
		//this.Refresh();
	}

	public void tbspeed_cr(object sender, EventArgs e) {
		oldbpm=bpm;
	}

	public void tbspeed_okp(object sender, KeyEventArgs e) {
		string s=e.KeyCode.ToString();
		if (q.isNaturalEntering(s)==false) {tbspeed.Text=""; }
		if (s.Equals("Enter")) trspeed.Select();
	} 

	public void tbspeed_lf(object sender, EventArgs e) {
		if (tbspeed.Text.Length<1) tbspeed.Text=oldbpm.ToString();
		int i=Int32.Parse(tbspeed.Text);
		if (i>=trspeed.Minimum && i<=trspeed.Maximum) {	bpm=i;	trspeed.Value=bpm; mytimerreset();}
		else tbspeed.Text=oldbpm.ToString();
	}
	
//			try { i=Int32.Parse(tbspeed.Text); }
//			catch(Exception ee) { q.mm("you failed!!!! -"+ee.Message); return; }
//			
//		if (i>0 && i<240) {
//			bpm=i;
//			trspeed.Value=bpm;
//		}
//		//q.mm(i);
//	}

	private void troctave_Scroll(object sender, System.EventArgs e){ 
		octpi=troctave.Value+1;
	}
	
	private void trbars_Scroll(object sender, System.EventArgs e){ 
		barsdisplay=(byte)(trbars.Value+1);
		mytimerreset();
		this.Refresh();
	}
	
	private void cbgroup_SIC(object sender, System.EventArgs e){
		int gr=cbgroup.SelectedIndex;
		cbinstrum.Items.Clear();
		if (gr>0) { // piano...................................................
			int gg=gr-1;
			cbinstrum.Items.AddRange(new object[] {
				inst[gg,0],inst[gg,1],inst[gg,2],inst[gg,3],inst[gg,4],inst[gg,5],
				inst[gg,6],inst[gg,7]});
			cbinstrum.SelectedIndex=0;
			pgm=(byte)((gr-1)*8+0); tb1.Text=pgm.ToString();
			octpi=lastinstoctpi; keypixpi=lastinstkeypixpi;
			chn=0; cbinstrum.Visible=true; 
			lbkbzoom.Visible=true; trkbzoom.Visible=true; 
			lboctave.Visible=true; troctave.Visible=true; 
			lbbars.Visible=false; trbars.Visible=false; 
			tbrytname.Visible=false; btnew.Visible=false;
			btload.Visible=false; btsave.Visible=false;
			this.Refresh();
		} else { // rytm.......................................................
			midionline.play1tone((byte)0); // last tone off
			chn=9; // ryt-channel
			lastinstoctpi=octpi; lastinstkeypixpi=keypixpi;
			octpi=0; keypixpi=13; 
			lbkbzoom.Visible=false; trkbzoom.Visible=false;
			lboctave.Visible=false; troctave.Visible=false; tbrytname.Visible=true;
			lbbars.Visible=true; trbars.Visible=true; 
			lastdrum=31; tb3.Text="31"; tbrytname.Text=rytinst[31];
			cbinstrum.Visible=false; btnew.Visible=true;
			btload.Visible=true; btsave.Visible=true;
			
			this.Refresh();
		}
		//q.mm("box1="+gr);
	}

	private void cbinstrum_SIC(object sender, System.EventArgs e){ 
		int gg=cbgroup.SelectedIndex; if (gg<0) gg=1; 
		int sou=cbinstrum.SelectedIndex;
		pgm=(byte)((gg-1)*8+sou); tb1.Text=pgm.ToString();
	}

	private void f1_Click(object sender, System.EventArgs e){ // sender and e are tradition
		int mpx=MousePosition.X-this.Left;
		int mpy=MousePosition.Y-this.Top-20;
		//q.mm("this.Top="+this.Top);	q.mm(mpx.ToString()+"/"+mpy.ToString());
		int mpx1=mpx-xpi-4; int mpy1=mpy-ypi;
		
		if (mpx1>=0 && mpx1<=widpi && mpy1>=0 && mpy1<=heipi+0)	{// it's a keyboard-event...................
			int blackkey=0;
			int t=mpx1 / keypixpi + octpi*12; 
			int m= t % 12; 
			if (m==1 || m==3 || m==6 || m==8 || m==10) { //q.mm("m="+m);
				blackkey=1;
				// under the black keys: left half = -1 / reight half=+1 ..............
				if (mpy1>heipi/5*4) { //q.mm(mpx % keypixpi);
					blackkey=0; //q.mm(blackkey);
					if (mpx1 % keypixpi < keypixpi/2) t--;
					else t++;
				}
			}
			keynr=t-octpi*12;
			// keyboard only for drums.........
			if (chn==9) {
				t=t+2*12+3; // rytm begins at D# third octave
				//q.mm(t);
				tbrytname.Text=rytinst[t];
			} else tbrytname.Text="";

			// keyboard for both (drums and piano).................................................
			midionline.play1tone((byte)t);
			lastdrum=(byte)t;
			tb3.Text=t.ToString();
			drawnowkey(xpi+keynr*keypixpi+keypixpi/2-5,ypi+heipi/5*3, blackkey);// set finger-print........
			//q.mm(keynr);
		
		} // end keyboard-event

		
		// drums-workbench-sound.....................................
		int mpx2=mpx-xwoe-4; int mpy2=mpy-ywo;
		if (mpx2>0 && mpx2<prewo && mpy2>0 && mpy2<=heiwo)	{
			colwo=mpx2 / colpix ; linwo=mpy2 / linheiwo;
			bool found=false;
			for (int i=0;i<linswo ;i++ ) if (rychoi[i]==lastdrum) found=true;
			if (found==false) drawrytlin(linwo);
			else MessageBox.Show("this rytm-sound is already set!","Juno"); 
			//this.MouseDown += new MouseEventHandler(altergraph);
		}

		// drums-workbench-tone.....................................
		if (mpx2>prewo && mpx2<widwo && mpy2>0 && mpy2<=heiwo)	{
			colwo=(mpx2-prewo) / colpix ; linwo=mpy2 / linheiwo;
			if (rychoi[linwo]!=0) drawryttag(linwo,colwo);
		}
		
		// sound-off.......................
		if (mpx>=xpi+widpi-40 && mpx <=xpi+widpi-40+32 && mpy>=ypi+heipi+8 && mpy<=ypi+heipi+8+32) {
			midionline.play1tone(0);
		}
	}
	
	//internal void altergraph(object sender,MouseEventArgs e) {
	internal void drawrytlin(int li) {
		Graphics g = this.CreateGraphics();
		//Rectangle ra=new Rectangle(e.X-5,e.Y-5,30,30);
		rychoi[li]=lastdrum; 
		string s=rytquick[lastdrum]; if (s.Length==0) s=lastdrum.ToString(); 
		g.FillRectangle(sbpaper, new Rectangle(xwoe,ywo+li*linheiwo+2,prewo, linheiwo-2));
		g.DrawString(s,myfont,sbblack,xwoe-1,ywo+li*linheiwo+2);
		//this.Invalidate(ra);
		////this.Refresh();
		g.Dispose();
	}
	
	internal void drawryttag(int li, int co) {
		Graphics g = this.CreateGraphics();
		Rectangle ra=new Rectangle(prewo+(co+1)*colpix+1,ywo+li*linheiwo+4,6,8);
		//q.mm("col="+co+"  ryt="+ryt[li,co]);
		if (ryt[li,co]==0) { // set................
			ryt[li,co]=1;
			g.FillEllipse(sbred,ra);
			midionline.play1tone((byte)rychoi[li]);
		} else {
			ryt[li,co]=0;
			g.FillEllipse(sbpaper,ra);
			midionline.play1tone((byte)27+2);
		}
		//g.DrawEllipse(pen2,e.X-5,e.Y-5,15,15);
		//this.Invalidate(ra);
		////this.Refresh();
		g.Dispose();
	}

	internal void drawnowkey(int x7, int y7, int blackkey) {
		Graphics g = this.CreateGraphics();
		SolidBrush sbtemp=new SolidBrush(q.color_back);
		// delete old fingerprint..................
		if (xlastkey>0) {
			if (colorlastkey==0) sbtemp=sbivory; else sbtemp=sbblack;
			tb1.Text=colorlastkey.ToString();
			g.DrawString("o",new Font("Verdana",8), sbtemp, xlastkey,y7);
		}
		// save new key for deleting later......
		xlastkey=x7; colorlastkey=blackkey;
		// new fingerprint ...........
		if (blackkey==0) sbtemp=sbblack; else sbtemp=sbivory;
		g.DrawString("o",new Font("Verdana",8), sbtemp,x7,y7);
		g.Dispose();
	}

	internal void f1_paint(object sender,PaintEventArgs e) {
 		Graphics g = e.Graphics;
		int k=keypixpi; int keys=widpi / keypixpi;
		
//		// draw some outfit-lines............
//		g.DrawLine(new Pen(q.color_high,3),0,90,this.Width/8*2,this.Height);
//  	g.DrawLine(new Pen(q.color_high,1),0,60,this.Width/8*2+6,this.Height);
//		g.DrawLine(new Pen(q.color_high,3),150,5,this.Width/8*3+20,this.Height);
//		g.DrawLine(new Pen(q.color_high,1),162,5,this.Width/8*3+26,this.Height);
//		g.DrawLine(new Pen(q.color_high,3),this.Width,70,this.Width/8*6,this.Height);
//  	g.DrawLine(new Pen(q.color_high,1),this.Width,40,this.Width/8*6-6,this.Height);
//		g.DrawLine(new Pen(q.color_high,3),this.Width-150,5,this.Width/8*5-20,this.Height);
//  	g.DrawLine(new Pen(q.color_high,1),this.Width-162,5,this.Width/8*5-26,this.Height);
		
		// form-background...........................
		Image im1 = new Bitmap (progdir+@"\airloop13.jpg");
		Image im2 = new Bitmap (progdir+@"\back01.jpg");
		if (skin==1) {
			g.DrawImage (im1, ClientRectangle);
			im1.Dispose();
		}
		if (skin==2) {
			g.DrawImage (im2, ClientRectangle);
			im2.Dispose();
		}
		
		// image for off-button...............
		Image im7 = new Bitmap (progdir+@"\soundoff.bmp");
		Rectangle ra8=new Rectangle(xpi+widpi-42, ypi+heipi+6,36,36);
		g.FillRectangle(sbcontrol,ra8);
		Rectangle ra7=new Rectangle(xpi+widpi-40, ypi+heipi+8,32,32);
		g.DrawImage (im7, ra7);
		im7.Dispose();
		
		// draw background for main-button-group.....
		Rectangle ramg=new Rectangle(4,2,60,ypi+heipi+2*32-10);
		g.FillRectangle(sbgreen,ramg);
		
		//draw piano ....
  	Rectangle ra=new Rectangle(xpi,ypi,widpi,heipi);
  	g.DrawRectangle(penblack,ra);
		Rectangle rai=new Rectangle(xpi+0,ypi+0,widpi-0,heipi-0);
		g.FillRectangle(sbgray,rai);
		for (int i=0;i<keys;i++ ) {
			int m= i % 12;
			if (m==0 || m==2 || m==4 || m==5 || m==7 || m==9 || m==11) {
				Rectangle ran=new Rectangle(xpi+2+i*k,ypi,k-2,heipi);
				g.FillRectangle(sbivory,ran); }
			else {
				Rectangle ran=new Rectangle(xpi+i*k+2,ypi,k-3,heipi/5*4);
				g.FillRectangle(sbblack,ran);
				// version a).......
				//Rectangle rao=new Rectangle(xpi+i*k+2,ypi+heipi/5*4+2,k-2,heipi/5*1);
				//g.FillRectangle(sbivory,rao);
				// version b)......
				Rectangle rao1=new Rectangle(xpi+i*k,ypi+heipi/5*4+2,k/2-1,heipi/5*1);
				g.FillRectangle(sbivory,rao1);
				Rectangle rao2=new Rectangle(xpi+i*k+k/2+1,ypi+heipi/5*4+2,k,heipi/5*1);
				g.FillRectangle(sbivory,rao2);
			}
			if (m==0) g.DrawString("o",new Font("Verdana",8), sbblack,xpi+i*k,ypi);
		}
	}

	internal void drawrytNames(object sender,PaintEventArgs e) {
		// quicknote for best sounds if channnel=9 (for rytm) ...............
 		Graphics g = e.Graphics;
		if (chn==9) {
			int d=0;
  		foreach (string rq in rytquick) {
  			if (rq.Equals("")==false)
  				//g.RotateTransform(90); 
  				g.DrawString(rq,myfont7, sbgreen,xpi+(d-27)*keypixpi+1,
  				             ypi+heipi/4+(d % 2)*10-1-2);
  			d++;
  		}
		}
	}

	internal void drawMyGrid(object sender,PaintEventArgs e) {
		if (chn==9) {
			Graphics g = e.Graphics;
			penbluedot.DashStyle=System.Drawing.Drawing2D.DashStyle.Dot;
			penbluedash.DashStyle=System.Drawing.Drawing2D.DashStyle.Dash;
			Rectangle ra=new Rectangle(xwoe,ywo-12,widwo+prewo, heiwo+12);
			g.FillRectangle(sbpaper,ra);
			for (int i=0;i<colswo;i++ ) { // vertical lines ......................
				if (i % tpb==3 || i % tpb==6 || i % tpb==9) {
					penx=penbluedash;
					g.DrawString("y", myfont ,sbivory,i*colpix+xwoi,ywo-12);
				} 
				else penx=penbluedot;
				g.DrawLine(penx,i*colpix+xwoi,ywo,i*colpix+xwoi,ywo+heiwo);
				if (i % tpb ==0) g.DrawString((i / tpb +1).ToString(), myfont ,sbivory,i*colpix+xwoi,ywo-12);
				//if (i % 12==0) g.DrawLine(pen3,i*colpix+xwoi,ywo,i*colpix+xwoi,ywo+heiwo);
				if (i % tpb==0) g.DrawLine(peninput,i*colpix+xwoi,ywo-12,i*colpix+xwoi,ywo+heiwo);
			}
			int rr=0; string ss=""; // horizontal lines ...............
			for (int i=0;i<linswo;i++ ) {
				g.DrawLine(peninput,xwoe,ywo+i*linheiwo,prewo+widwo+8,ywo+i*linheiwo);
				rr=rychoi[i]; 
				if (rr!=0) { // names of drumsounds.....
					ss=rytquick[rr]; if (ss.Length==0) ss=rr.ToString();
					g.DrawString(ss, myfont, sbblack,
					xwoe-1,ywo+i*linheiwo+2); 
				}
			}
			
			// set metronome-tags to ryt-array.........
			for (int i=0;i<colswo;i++ )ryt[0,i]=0; 
			for (int i=0;i<barsdisplay;i++ )ryt[0,i*tpb]=1; 
			ryt[1,0]=1; // beginning-beat

			// draw ryt-tags.....
			for (int li=0; li<linswo;li++ ) {
				for (int co=0; co<colswo;co++ ) {
					int t=ryt[li,co];
					if (t>0) g.FillEllipse(sbred,prewo+(co+1)*colpix+1,ywo+li*linheiwo+4,6,8);
				}	
			}
		} // end if (ch==9)
	}
	
	internal static void mytimerreset() {
		tim1=System.DateTime.Now.Ticks; checks=0; tic=0;
	}

	void initInstNames() {
		string s =""; 
		string sgg="DRUMS,PIANO,CHROM-PERCUS,ORGAN,GUITAR,BASS,STRINGS,ENSEMBLE,"
 			+"BRASS,REED,PIPE,SYNTI-LEAD,SYNTI-PAD,SYNTI-FX,ETHNIC,PERCUSSIVE,SPECIAL-FX";
 		s=sgg; 
    char[] separators = {','};
    igroup = s.Split(separators);

		string[] sg=new string[16]; 
		sg[0]="Grand-Piano,Bright-Piano,EGrand-Piano,Honky-tonk,E-Piano1,E-Piano2,Harpsichord,Clavinova";
		sg[1]="Celesta,Glockenspiel,MusicBox,Vibraphone,Marimba,Xylophone,Tubular-Bells,Dulcimer";
		sg[2]="Drawbar-Organ,Perc.Organ,Rock-Organ,Church-Organ,Reed-Organ,Accordion,Harmonica,TangoAccord";
		sg[3]="Nylon-Guitar,Steel-Guitar,Jazz-Guitar,Clean-Guitar,Muted-Guitar,Overdrive-Git,Distort-Git,GitHarmonics";
		sg[4]="A-Bass,Fingered-Bass,Picked-Bass,Fretless-Bass,Slap-Bass1,Slap-Bass2,Synth-Bass1,Synth-Bass2";
		sg[5]="Violin,Viola,Cello,Contrabass,TremoloString,PizzictoString,Harp,Timpani"; 
		sg[6]="Strings1,Strings2,Syn-Strings1,Syn-Strings2,choir-Aahs,Voice-Oohs,Synth-Voice,Orchestra-Hit";
		sg[7]="Trumpet,Trombone,Tuba,Muted-Trumpet,French-Horn,Brass-Section,Synth-Brass1,Synth-Brass2";
		sg[8]="Soprano-Sax,Alto-Sax,Tenor-Sax,Baritone-Sax,Oboe,English-Horn,Bassoon,Clarinet";
		sg[9]="Piccolo,Flute,Recorder,Pan-Flute,Blown-Bottle,Shakuhachi,Whistle,Ocarina";
		sg[10]="Square-Lead,Saw.-Lead,Calliope-Lead,Chiff-Lead,Charang-Lead,Voice-Lead,Fifth-Lead,Bass-Lead"; 
		sg[11]="New-Age-Pad,Warm-Pad,Polysynth-Pad,choir-Pad,Bowed-Pad,Metallic-Pad,Halo-Pad,Sweep-Pad";
		sg[12]="Rain,Soundtrack,Crystal,Atmosphere,Brightness,Goblinswo,Echoes,Sci-Fi";
		sg[13]="Sitar,Banjo,Shamisen,Koto,Kalimba,Bagpipe,Fiddle,Shanai";
		sg[14]="Tinkle-Bell,Agogo,Steel-Drums,Woodblock,Taiko-Drum,Melodic-Drum,Synth-Drum,Reverse-Cymbal";
		sg[15]="GitFretNoise,Breath-Noise,Seashore,Bird-Tweet,Telephone,Helicopter,Applause,Gunshot";
		for (int i=0;i<16 ;i++ ) {
			s=sg[i];
			string[] instrum = s.Split(separators);
			int k=0; foreach (string sub in instrum) {inst[i,k]=sub; k++;}
		}
		s=",,,,,,,,,,,,,,,,,,,,,,,,,,,"
		  +"High-Q,Slap,Scratch-Push,Scratch-Pull,Sticks,Square-Click,Metro-Click,Metro-Bell,"
			+"A-BassDrum,Bass-Drum1,Side-Stick,A-Snare,Hand-Clap,Electric-Snare,Low-Floor-Tom,"
			+"Closed-HighHat,High-Floor-Tom,Pedal-HighHat,Low-Tom,Open-HighHat,Low-Mid-Tom,"
			+"High-Mid-Tom,Crash-Cymbal1,High-Tom,Ride-Cymbal1,Chinese-Cymbal,Ride-CymbalBell,"
			+"Tambourine,Splash-Cymbal,Cowbell,Crash-Cymbal2,Vibraslap,Ride-Cymbal2,"
			+"High-Bongo,Low-Bongo,Mute-High-Bongo,Open-High-Bongo,Low-Conga,High-Timbale,"
			+"Low-Timbale,High-Agogo,Low-Agogo,Cabasa,Maracas,Short-Whistle,Long-Whistle,"
			+"Short-Guiro,Long-Guiro,Claves,High-Woodblock,Low-Woodblock,Mute-Cuica,Open-Cuica,"
			+"Mute-Triangle,Open-Triangle,Shaker,Jingle-Bells,Belltree,Castanets,Mute-Surdo,Open-Surdo,"
			+"Casta-close";
		rytinst = s.Split(separators);
		for (int i=0;i<maxdrumsounds ;i++ ) {rytquick[i]="";}
		rytquick[31]="st"; rytquick[36]="bs"; rytquick[37]="si"; rytquick[39]="cp";
		rytquick[40]="sn"; rytquick[42]="cl"; rytquick[44]="pd"; rytquick[45]="lt"; 
		rytquick[46]="op"; rytquick[48]="mt"; rytquick[49]="cy"; rytquick[50]="ht"; 
		rytquick[51]="ri"; rytquick[53]="cs"; rytquick[54]="tc"; rytquick[56]="cw"; 
		rytquick[69]="sa"; rytquick[81]="tr"; rytquick[83]="to"; 
		rytquick[70]="ma"; rytquick[85]="ca";
	
		// predefined rytm-lines : sound-setting....
		rychoi[0]=70;rychoi[1]=85;
		rychoi[2]=36;rychoi[3]=40;
		rychoi[4]=44;rychoi[5]=46;rychoi[6]=49; rychoi[7]=51;
		for (int i=8;i<linswo ;i++ ) rychoi[i]=0;
		//q.mm(rytquick[rychoi[0]]);
		
		// reset all ryt-entries..........
		for (int i=0;i<linswo ;i++ ) {
			for (int k=0;k<colswo ;k++ )	
				ryt[i,k]=0;
		}
	}
	

	[STAThread]
	public static void Main(string[] args)
	{
		Application.Run(new MainForm());
	}
	
}	// end mainframe	



internal class midireplay {
	internal static QuartzTypeLib.FilgraphManager grman = new 	QuartzTypeLib.FilgraphManager();
	internal static QuartzTypeLib.IMediaControl mc=(QuartzTypeLib.IMediaControl)grman;
	internal static string lastmidnam=""; 
	internal static bool midireplaying=false;
	
	internal static void playmid(string finam, Midmod mimo) {
		// try to disable any other usage of midi-device by midionline
		if 	(midionline.lasttone!=0) {
			midionline.play1tone(0);
			MidiPlayer.CloseMidi(); 
		} 

		if (finam.Equals(lastmidnam)==false) {
			if (mc != null) mc.Stop();
			mc.RenderFile(finam); lastmidnam=finam; 
		}
		if (mimo==Midmod.run) {midireplaying=true; mc.Run();}
		if (mimo==Midmod.stop){midireplaying=false;  mc.Stop();}
		if (mimo==Midmod.rewind) {mc.Stop(); mc.RenderFile(finam); midireplaying=true; mc.Run();}
	}
} // end midireplay


internal class midionline {
		//MidiPlayer.Play(new NoteOn(0, GeneralMidiPercussion.CrashCymbal1,127));
	internal static byte lasttone=0; 
	internal static void play1tone(byte tt) {
		if (lasttone!=0) {MidiPlayer.Play(new NoteOff(0, MainForm.chn,lasttone,127));}

		//try to disable any other usage of midi-device by midireplay
		if (midireplay.midireplaying) {
			midireplay.playmid(midireplay.lastmidnam,Midmod.stop);
			//midireplay.lastmidnam="";
		}
		
		if (lasttone==0) {			MidiPlayer.OpenMidi();		}
		if (tt==0) MidiPlayer.CloseMidi();
		else {
			if (MainForm.pgm!=MainForm.oldpgm || lasttone==0) {
				MidiPlayer.Play(new ProgramChange(0,MainForm.chn,MainForm.pgm));
			}
			MidiPlayer.Play(new NoteOn(0, MainForm.chn,tt,127));
		}
		lasttone=tt;
		MainForm.oldpgm=MainForm.pgm; MainForm.tb2.Text=MainForm.chn.ToString();
	}
	
} // end midionline


class q {
	public static void mmbase(string s) {
		DialogResult dr=MessageBox.Show("go on?",s,MessageBoxButtons.YesNoCancel);
		if (dr==DialogResult.No) Application.Exit();	}
		public static void mm(string s) {mmbase(s);}
		public static void mm() {mmbase("");}
		public static void mm(int i) {mmbase(i.ToString());}
		public static void mtoprog() {mmbase("sorry, this part of the program is still not available");}
	public static string fileselect(string filter) {
		OpenFileDialog openFileDialog = new OpenFileDialog();
		openFileDialog.Filter = filter; // "Media Files|*.mid;*.mp3|All Files|*.*";
		openFileDialog.ShowDialog();
		return openFileDialog.FileName;	}
	public static string fileselect() {
		OpenFileDialog openFileDialog = new OpenFileDialog();
		openFileDialog.Filter = "All Files|*.*";
		openFileDialog.ShowDialog();
		return openFileDialog.FileName;
	}
	public static string savefileselect(string filter) {
		SaveFileDialog sfd = new SaveFileDialog();
		sfd.Filter = filter; // "Media Files|*.mid;*.mp3|All Files|*.*";
		sfd.ShowDialog();
		return sfd.FileName;	}
	public static string savefileselect() {
		SaveFileDialog sfd = new SaveFileDialog();
		sfd.Filter = "All Files|*.*";
		sfd.ShowDialog();
		return sfd.FileName;
	}
	public static bool isNaturalEntering(string s) {
		string snumbers=" D0 D1 D2 D3 D4 D5 D6 D7 D8 D9 "
			+"NumPad0 NumPad1 NumPad2 NumPad3 NumPad4 NumPad5 NumPad6 NumPad7 NumPad8 NumPad9 "
			+"Left Right Up Down Delete Back Home End Enter F1 Escape ";
		//System.Console.WriteLine("Your input= {0} ",s);
		if (s.Length>1 && snumbers.IndexOf(s)>0) return true; else return false;
	}
	public static Color color_back =Color.FromArgb(0,0,96); // darkest blue
	public static Color	color_paper=Color.FromArgb(112,176,208); // light blue, touch to gray
	public static Color	color_control=Color.LightSteelBlue;
	public static Color	color_input=Color.FromArgb(255,250,210); // ivory (beige-yellow)
	public static Color	color_mid=Color.FromArgb(255,255,188); // midgreen
	public static Color	color_high=Color.FromArgb(255,230,140); // lightyellow (highlighted)
	
} // end q


} // end namespace


